﻿;;;-*- Mode: Lisp; Package: "OMAS" -*-
;;;===============================================================================
;;;15/01/29
;;;                L O A D - O M A S  (file omas-load.lisp)
;;;
;;;                Copyright JP BArthes @UTC, 2007-2012
;;;
;;;===============================================================================

#| This file loads the OMAS environment (Open Multi-Agent System).
   Loads all the OMAS files and opens the init window.
   Can be used by MCL OR ACL.

2007
 0319 removing MOSS from the load function (must be preloaded)
 0623 introducing default for the OMAS init window
 0823 selecing the init window in ACL
2008
 0521 putting an optional make-package
 0621 === v7.0.0
2009
 0512 adding the possibility of compiling files with specific external format
2010
 0325 saving omas-directory into *omas*
 0618 outsourcing part of load-omas into load-omas-file and exporting it
 0624 small cleanup removing old code
2011
 1211 introducing the possibility of bypassing the init window by adding a line
      to the configuration file: AUTOMATIC = T
      However, init window will appear if ALT key is presse
 1217 new format of *omas-parameters*, open graphics window on request
2012
 0103 improving the error message when no application is present
 0506 modifying the use of INIT window and automatic loading
      creating LOAD-APPLICATION-OR-OPEN-INIT-WINDOW, remove LOAD-DIRECTLY
2014
 0308 -> utf8
2015
 0129 adding the load-application-file function
|# 

(eval-when (:compile-toplevel :load-toplevel)
  (unless (find-package :omas) (make-package :omas :use '(:cl #+MCL :ccl)))
  )

(in-package :omas)

(eval-when (:compile-toplevel :load-toplevel :execute)
  (export '(load-omas-file load-application-file)))

;;;===============================================================================

#-MOSS
(error ";***** You cannot load OMAS without MOSS **** ~&~
        To load OMAS use load-omas-moss file in the OMAS-MOSS folder")

;;; kludge to avoid removing *omas-version-number* from everywhere
(defParameter *omas-version-number* *omas-moss-version*)

;;; ------------------------------------------------------------------------------
;;; if no file was called before this one, then we must get the version info
;;; they are in the OMAS-MOSS directory one level up the file tree

#-OMAS
(progn
  (let ((init-file 
         (make-pathname
          :device (pathname-device *load-pathname*)
          :host (pathname-host *load-pathname*)
          :directory (butlast (pathname-directory *load-pathname*))
          :name "OMAS-version-info"
          :type "lisp")))
    (format t "~&init-file: ~S" init-file)
    (load init-file))
  ;; record moss
  (pushnew :omas *features*)
  )

;;; ------------------------------------------------------------------------------
;;; omas represents the main directory, i.e. *omas-load-pathname* strips 
;;; *load-pathname* of the cfsl or cfsl subdirectory when loading from compiled
;;; load files. Indeed in that case:
;;; *load-pathname* = 
;;;    #P"Thor:Users:barthes:MCL:OMAS-MOSS:OMAS-v5.2:cfsl:5-omas-load-M.cfsl"
;;; or "C:\\..."
;;; *omas-directory-string* = "Thor:Users:barthes:MCL:OMAS MOSS:OMAS-v5.2:"
;;; In other words *omas-directory-string* points at the OMAS directory. The text files 
;;; are then in the ":omas source" folder, and compiled files in the :cfsl folder.
;;; Applications are in the :application folder.

(defParameter *omas-directory-string* 
  (let ((directory (pathname-directory *load-pathname*)))
    (make-pathname
     :device (pathname-device *load-pathname*)
     :host (pathname-host *load-pathname*)
     :directory (if (member (car (last directory)) '("fasl" "cfsl") :test #'string=)
                  (butlast directory)
                  directory))))

;(print `("*omas-directory-string*" ,*omas-directory-string*))

;;; ------------------------------------------------------------ COMPILE-IF-NEEDED
;;; we want to compile a file, when we have no compiled file or when the compiled
;;; file is older than the source file.

(defun compile-if-needed (source compiled &optional external-format)
  "Compiles source if not compiled, if changed.
Arguments:
   source: file pathname of source code
   compiled: file pathname of compiled result
   external-format (optional): format of source file, e.g. :utf-8
Result:
   unimportant"
  (let ()
    (when (or (not (probe-file compiled))
              (< (file-write-date compiled)
                 (file-write-date source)))
      (apply #'compile-file source :output-file compiled :verbose t
             (and external-format (list :external-format external-format))))))

;;;-------------------------------------------------------------- LOAD-APPLICATION
;;; the function is used to load the application directly and also to load the
;;; application from the init window. In the first case, *omas* must be initialized
;;; prior to call this function (done in load-application-or-open-init-window).

(defun load-application (&optional (folder-name "applications"))
  "central function to load a full application, once the global parameters have ~
   been extracted from the parameters file and stored into *omas*.
Arguments:
   :folder-name (opt): name of folder containing the application files 
                       default \"applications\"
Return:
   T if load was successful, NIL otherwise"
  
  (let ((site-name (site-name *omas*))
        (appli/coterie-name (coterie-name *omas*))
        file-name)
    
    ;; build a local user name to go with the control panel, e.g. :<ODIN-USER>
    (setf (local-user *omas*)
          (intern (string-upcase (format nil "<~A-USER>" site-name)) 
                  :keyword))
    
    ;;============================================================================
    ;;                  Get list of agents in the application
    ;;============================================================================
    
    ;; build up pathname for the application agent.lisp file    
    (setq file-name (make-application-file-pathname     
                     *omas-directory-string* folder-name     
                     site-name appli/coterie-name "agents"))    
    ;(format *debug-io*  "~&=== oi-load-on-click; agents.lisp: ~S" file-name)    
    ;; try to load agents file recompiling t if needed    
    (unless (compile-and/or-load file-name #-MCL :external-format #-MCL :utf-8)    
      (warn "Application ~A:~A not present in ~S folder.~2%
*** To change the application if the green init window is not present, exit from ~
the system, press the SHIFT key and hold it while the system is restarted. ~
This will trigger the green init menu for selecting a different application.***"
        site-name appli/coterie-name folder-name)    
      (%beep)
      (return-from load-application nil))
    
    ;; compute the application directory pathname from agents file-name
    ;; and save it for later user by make-agent JPB0912
    (setf (omas-application-directory *omas*) 
          (make-pathname :directory (pathname-directory file-name)))
    
    ;; tell user that everything is OK, print local agent names    
    ;(format *debug-io* "~&;oi-load-on-click /local-coterie-agents: ~S" 
    ;  omas::*local-coterie-agents*)   
    
    ;;============================================================================
    ;;                               load agents
    ;;============================================================================
    
    ;; load each agent files    
    (dolist (agent *local-coterie-agents*)       
      ;; cook up agent file-pathname    
      (setq file-name (make-application-file-pathname     
                       *omas-directory-string* folder-name    
                       site-name appli/coterie-name     
                       (symbol-name agent))) 
      ;(format t "~&; oi-load-on-click /loading agent: ~S" agent)
      (load-agent :agent-file-pathname file-name)
      )
    
    ;;============================================================================
    ;;                            load message file
    ;;============================================================================
    ;; load message file if any (don't complain if none)    
    (setq file-name (make-application-file-pathname     
                     *omas-directory-string*  folder-name    
                     site-name appli/coterie-name "Z-messages"))    
    ;(format *debug-io*  "~&;oloc; source 3: ~S" file-name)    
    (compile-and/or-load file-name #-MCL :external-format #-MCL :utf-8)
    
    ;;============================================================================
    ;;                       load manager file (not available)
    ;;============================================================================
    ;; load manager file (use load-manager from the load file)
    (load-manager)
    
    ;;============================================================================
    ;;                            create control panel
    ;;============================================================================
    (make-omas-control-panel)
    
    ;;============================================================================
    ;;                               net control
    ;;============================================================================
    
    ;; keep messages local when broadcast net address is blank    
    (setf (net-broadcast *omas*)
          (if (string-equal (local-broadcast-address *omas*) "") nil t))
    
    ;; if using the net, then initialize
    (when (net-broadcast *omas*)    
      ;; This start a receiving process *omas-net-receive*    
      ;; and a dispatching process *omas-net-dispatch* that delivers the messages    
      ;; to the local agents    
      (omas-net-initialize))
    
    ;; here the policy would be that we enter a single event processing loop...    
    ;(%window-close *init-window*)
    
    (when (equal appli/coterie-name "NULL")
      (setq *package* (find-package #+MCL :cl-user #-MCL :cg-user)))
    
    ;;==============================================================================
    ;;      if graphics-window is T open gaphics window
    ;;==============================================================================
    
    (if (graphics-window *omas*)
        (let ((spy-key (key (symbol-value (spy-name *omas*)))))
          ;; if agents were not selected select them all
          (unless (names-of-agents-to-display *omas*)
            (setf (names-of-agents-to-display *omas*) 
              (mapcar #'car (local-agents *omas*))))
          (%send-message (make-instance 'message :type :sys-inform :to spy-key
                           :action :enable)
                         t) ; indicates local message
          ))

    ;;==============================================================================
    ;;                 check init function (from agents.lisp)
    ;;==============================================================================
    (if (fboundp 'init) (funcall 'init))
    
    :done)
  t)

;;;--------------------------------------------------------- LOAD-APPLICATION-FILE

(defun load-application-file (filename &optional (utf8-flag :utf-8))
  "given a string specifying the application file, loads it compiling it if needed. Uses ~
   (omas-application-directory *omas*) locates the directory containing the file.
Argument:
   filename: a string
   utf8-flag (opt): if :utf-8 (default) indicates that the file is utf8 encoded
Return:
   :done"
  (let* ((dir-path (omas-application-directory *omas*))
         (source (merge-pathnames (string+ dir-path filename ".lisp")))
         (compiled (make-pathname
                    :device (pathname-device dir-path)
                    :host (pathname-host dir-path)
                    :directory `(,@(pathname-directory dir-path) "FASL")
                    :name filename
                    :type "fasl"
                    )))
    
    ;(format t "~2%;---------- load-application-file")
    ;(format t "~%;--- *package*: ~S" *package*)
    ;(format t "~%;--- source:~%  ~S" source)
    ;(format t "~%;--- compiled:~%  ~S" compiled)
    #+compiler ; when compiler is there, compile file if needed
    (cond
     ;; if source file is there, check for possible recompiling
     ;; since patches are available as source file, they will be recompiled
     ;; if needed
     ((probe-file source)
      (compile-if-needed source compiled utf8-flag)
      ;(format *debug-io* "~&Load-pathname 1: ~S" compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; if compiled file is there load it
     ((probe-file compiled)
      ;(format *debug-io* "~&Load-pathname 2: ~S" compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; otherwise complain
     (t (warn "~&File missing: ~S" filename)))
    
    ;; when compiler is not there (some distributed versions), load source
    ;; file if more recent than compiled file
    #-compiler
    (cond
     ((and (probe-file source) (probe-file compiled)
           (<= (file-write-date compiled)
               (file-write-date source)))
      (apply #'load source (and utf8-flag (list :external-format :utf-8)))
      )
     ;; if source file there, then it is more recent
     ((probe-file source)
      (apply #'load source (and utf8-flag (list :external-format :utf-8))))
     ;; if source not there, check compiled and load it
     ((probe-file compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; no file there
     (t (warn "~&File missing: ~S" filename)))
    :file-loaded))

;;;------------------------------------------ LOAD-APPLICATION-OR-OPEN-INIT-WINDOW

(defun load-application-or-open-init-window ()
  "record omas parameters into *omas* table and either load application if ~
   AUTOMATIC is not true and SHIFT key is not pressed, or open INIT window."
  (declare (special omas::*omas-parameters*))
  
  ;; check if must open INIT window
  (cond
   ;; if config file contains AUTOMATIC and ALT key is not pressed, load immediately
   ((and  
     (read-from-string 
      (caddr (assoc "AUTOMATIC" omas::*omas-parameters* :test #'string-equal))
      nil nil) ; returns nil if string is empty (EOF detected)
     (not (cg:key-is-down-p cg:vk-shift)))
    ;; now that OMAS is loaded, we can save parameters into *omas* table 
    (omas::save-omas-properties-to-globals omas::*omas-parameters*)
    ;; here we assume that the parameters file exists and do not save parameters
    ;; again
    
    ;; load application from applications folder or sample-applications folder
    (unless (or (load-application) 
                (load-application "sample-applications"))
      ;; if we could not load then open INIT window
      (omas::make-omas-init-window)
      (cg:set-foreground-window omas::*init-window*)))
   
   ;; otherwise, open the INIT window
   (t    
    (omas::make-omas-init-window)
    (cg:set-foreground-window omas::*init-window*)
    ))
  )
                                             
;;;----------------------------------------------------------------- LOAD-DIRECTLY
;;; load-directly is a quasi copy of the OI-load-on-click function

;;;(defun load-directly ()
;;;  "launches the application by loading files bypassing the init window. Uses the ~
;;;   *omas-parameters* list to save parameters into the global site structure first."
;;;  (declare (special *omas-parameters*))
;;;  ;; save SITE, COTERIE, PORT, BROADCAST-ADDRESS,...
;;;  (save-omas-properties-to-globals *omas-parameters*)
;;;  ;; now can load application, once the parameters were saved
;;;  (load-application)
;;;  )


;;; -------------------------------------------------------------------- LOAD-OMAS

(defUn load-omas ()
  "load OMAS as follows:
   - first compute the paths for the source and compiled files
   - if source is there, then check if different from the compiled version and if 
     the compiler is available:
     - if different or no compiled file, compile the source
     - then load the compiled file"
  (let (utf8-flag)
    (format t "~&;*** --- Loading OMAS --- ***")
    (dolist (filename *omas-file-names*)
      ;; adding a utf8 flag for taking care of encoding
      (setq utf8-flag (and (search "utf8" filename :test #'string-equal) :utf-8))
      (load-omas-file (format nil "~A-~A" *omas-prefix* filename) utf8-flag)
      )
    :OMAS-files-loaded))

;;;---------------------------------------------------------------- LOAD-OMAS-FILE

(defun load-omas-file (filename &optional utf8-flag)
  "given a string specifying the omas file, loads it compiling it if needed. Uses ~
   *omas-directory-string* to locate the directory containing the file.
Argument:
   filename: a string
   utf8-flag (opt): if true indicates that the file is utf8 encoded
Return:
   :done"
  (declare (special *omas-directory-string*))
  (let ((source (make-pathname
                 :device (pathname-device *omas-directory-string*)
                 :host (pathname-host *omas-directory-string*)
                 :directory (append (pathname-directory *omas-directory-string*)
                                    (list "OMAS SOURCE"))
                 :name filename
                 :type "lisp"
                 ))
        (compiled (make-pathname
                   :device (pathname-device *omas-directory-string*)
                   :host (pathname-host *omas-directory-string*)
                   :directory (append (pathname-directory *omas-directory-string*)
                                      (list "OMAS SOURCE")
                                      (list #+MCL "CFSL"
                                            #+MICROSOFT-32 "FASL"))
                   :name filename
                   :type #+MCL "cfsl" #+MICROSOFT-32 "fasl"
                   )))
    
    #+compiler ; when compiler is there, compile file if needed
    (cond
     ;; if source file is there, check for possible recompiling
     ;; since patches are available as source file, they will be recompiled
     ;; if needed
     ((probe-file source)
      (compile-if-needed source compiled utf8-flag)
      ;(format *debug-io* "~&Load-pathname 1: ~S" compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; if compiled file is there load it
     ((probe-file compiled)
      ;(format *debug-io* "~&Load-pathname 2: ~S" compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; otherwise complain
     (t (warn "~&File missing: ~S" filename)))
    
    ;; when compiler is not there (some distributed versions), load source
    ;; file if more recent than compiled file
    #-compiler
    (cond
     ((and (probe-file source) (probe-file compiled)
           (<= (file-write-date compiled)
               (file-write-date source)))
      (apply #'load source (and utf8-flag (list :external-format :utf-8)))
      )
     ;; if source file there, then it is more recent
     ((probe-file source)
      (apply #'load source (and utf8-flag (list :external-format :utf-8))))
     ;; if source not there, check compiled and load it
     ((probe-file compiled)
      (apply #'load compiled (and utf8-flag (list :external-format :utf-8))))
     ;; no file there
     (t (warn "~&File missing: ~S" filename)))
    :file-loaded))

        
  
;;;------------------------------------------------------------- MAKE-NEW-PATHNAME
;;; used by the initwindow callbacks in particular

(defun make-new-pathname (pathname &key device host directory name type version)
  "build a new pathname from a given one by supplying parts.
Arguments:
   pathname: pathname of a reference file
   device (opt): device
   host (opt): host
   directory (key): a list of directories
   name (key): filemname
   type (key): file type
   version (key): version if it applies
Return:
   updated pathname."
  (make-pathname
   :device (or device (pathname-device pathname))
   :host (or host (pathname-host pathname))
   :directory (or directory (pathname-directory pathname))
   :name (or name (pathname-name pathname))
   :type (or type (pathname-type pathname))
   :version (or version (pathname-version pathname))))

#|
? omas::*omas-directory*
#P"C:\\Program Files\\acl81-express\\OMAS-MOSS 7.0.5\\load-omas-moss.fasl"
? (make-new-pathname omas::*omas-directory* 
      :name "my-file"
      :type "cl")
#P"C:\\Program Files\\acl81-express\\OMAS-MOSS 7.0.5\\my-file.cl"
? omas::*omas-directory-string*
#P"C:\\Program Files\\acl81-express\\OMAS-MOSS 7.0.5\\OMAS\\"
? (make-new-pathname omas::*omas-directory-string* 
      :name "my-file"
      :type "cl")
#P"C:\\Program Files\\acl81-express\\OMAS-MOSS 7.0.5\\OMAS\\my-file.cl"
|#
;;;------------------------------------------------------------------ load-manager

(defun load-manager ()
  "this function is called after loading the application. Indeed, we must wait to
know the coterie name, since this is going to be the name of the manager"
  ;;;  (let (source fasl)
  ;;;      (setq fasl
  ;;;            (make-new-pathname 
  ;;;             *omas-load-pathname*
  ;;;             :directory (append (pathname-directory *omas-load-pathname*) '("fasl"))
  ;;;             :name (concatenate 'string *omas-prefix* "manager")
  ;;;             :type "fasl")
  ;;;            source (make-new-pathname 
  ;;;             *omas-load-pathname*
  ;;;             :directory (append (pathname-directory *omas-load-pathname*) '("source"))
  ;;;             :name (concatenate 'string *omas-prefix* "manager")
  ;;;             :type "lisp"))
  ;;;    ;(format *debug-io* "~&Load-pathname: ~S" source)
  ;;;    (cond
  ;;;     ((and (null source)(null fasl))
  ;;;      (warn "~&File manager not found"))
  ;;;     ((and source (not *distributed-version*))
  ;;;      (excl::compile-file-if-needed source :output-file fasl)
  ;;;      (load fasl))
  ;;;     ((load fasl)))
  ;;;    )
  :OMAS-manager-file-loaded)

;;; ------------------------------------------------------------------------------

(in-package :omas)

#+MCL
(progn
  ;;; MCL resize listener to fit under the graphics trace onto a large screen 
  ;;; (1200x800)
  ;;; first define a left screen offset
  ;;;  - 4 for normal screens
  ;(import '(cl-user::*screen-left-offset*))
  (defParameter *screen-left-offset* 4)
  )

;;; ------------------------------------------------------------------------------

(eval-when (compile load eval)
  (defVar *init-list* ()) ;contains the environment variables
  (defVar *local-coterie-agents* ())
  )

;; load files
(eval-when (load eval)
  #+MCL (unintern 'cl-user::omas)
  (load-omas))

;; launch init window, except if we have an automatic load in *omas-parameters*
;;; AUTOMATIC = T in the configuration file and ("AUTOMATIC" NIL NIL) in the
;;; omas::*omas-parameters* alist
;;; if the alt key is pressed, then we open the init window anyway
#+MCL
(eval-enqueue (omas::make-omas-init-window))
#+MICROSOFT-32
;;; here we should transfer parameters from *omas-paameters* to *omas* and
;;; load the application if AUTOMATIC is true and SHIFT key is not pressed, 
;;; or open the INIT window otherwise
(eval-when (:load-toplevel :execute)
  (load-application-or-open-init-window)
;;;  (cond
;;;   ;; if config file contains AUTOMATIC and ALT key is not pressed, load immediately
;;;   ((and  
;;;     (read-from-string 
;;;      (caddr (assoc "AUTOMATIC" omas::*omas-parameters* :test #'string-equal))
;;;      nil nil) ; returns nil if string is empty (EOF detected)
;;;     (not (cg:key-is-down-p cg:vk-shift)))
;;;    (load-directly))
;;;   ;; otherwise, open the init window
;;;   (t    
;;;    (omas::make-omas-init-window)
;;;    (cg:set-foreground-window omas::*init-window*)
;;;    ))
  )

(format t "~&;*** Agent platform OMAS v ~A loaded ***" omas::*omas-version-number*) 

:EOF