;;;-*- Mode: Lisp; Package: "CG-USER" -*-
;;;===============================================================================
;;;13/4/8
;;;            O M A S - I N S T A L L (install.lisp)
;;;
;;;             copyright JP Barthes @UTC, 2011-2013
;;;
;;;===============================================================================

#| This file contains functions for installing OMAS-MOSS. Only valid for ACL on 
Windows. The OMAS platform is installed for the first time. It simply overwrites
the startup.cl file in the ACL folder.

Installing can be used for upgrading a previous OMAS version, but in that case the
"applications" file must be transferred into the new OMAS Release folder replacing
the default applications file.

Installing a new version
------------------------
Perform the following steps:
1. Loading "OMAS-MOSS Release m".n.p (ACL 8.2)" folder into the ALLEGRO CL 8.2
   folder (recommended), 
2. Start Allegro CL with IDE, go to the File menu and click the "File/compile and
   load ..." entry,
3. Go to the OMAS-MOSS Release folder and select the "install.lisp" file
   The OMAS-MOSS platform will then load.
4. The rest should be automatic. For your information, it will create a startup.cl
   file in the ALLEGRO CL folder, which when you start the ACL next time over, 
   will load OMAS automatically.
   
Note: if a startup.cl file exists in the ACL folder, ACL will start and execute
whatever is in this file. Deleting this file allows starting in a clean Lisp 
environment.

2011
 1207 creation
2013
 0311 modifying startup.cl file to adapt to ACL 9.0 for automatic startup
 0408 correcting a bug in install and upgrade files
|#

;;; We stay in the CG-USR package for ACL
(in-package :cg-user)


;;;================================== Globals ====================================
;;; get new OMAS file folder name

(defparameter *omas-folder-name* (car (last (pathname-directory *load-pathname*))))

(defparameter *omas-load-file* (merge-pathnames "load-omas-moss" *load-pathname*))

;;;============================== macros =========================================
;;; none

;;;============================= functions =======================================

;;;------------------------------------------------------------------ OMAS-INSTALL

(defun omas-install ()
  "for installing OMAS the first time around."
  (declare (special *omas-folder-name* *omas-load-file*))
  ;; first get the pathname to a possible ACL startup file
  (let ((startup-file (merge-pathnames "startup.cl" (excl::get-home-location))))
        
    (cond
     ;; if there is no startup file in ACl make one
     ((null (probe-file startup-file))
      (create-install-startup-file *omas-folder-name*)) 
    ;; when startup.cl file exists asks the user if she wants to overwrite it
    ((y-or-n-p 
             "A startup.cl file already exists. Do you want to overwrite it? 
If not, then you will have to update the file manually to boot OMAS automatically.
If you don't know what a startup file is, then you want to answer yes, which ~
will overwrite it."
             )
     (create-install-startup-file *omas-folder-name*))
     ;; otherwise we don't do anything
     )
    ;; get allegrocache before launching OMAS/MOSS
    #+ALLEGRO-V9.0 (require :acache "acache-3.0.6.fasl")
    ;; start LISP
    (load *omas-load-file*)))

;;;----------------------------------------------------------- CREATE-STARTUP-FILE

(defun create-install-startup-file (folder-name)
  "creates a file in the ACL folder, provided that the OMAS folder is there"
  ;; create a simple startup file
  (with-open-file (ff "startup.cl" :if-does-not-exist :create
                      :if-exists :supersede
                      :direction :output)
    (format ff ";;; File for loading MOSS or OMAS automatically from init window")
    (format ff "~2%(defparameter *OMAS-folder* ~S)" folder-name)
    #+ALLEGRO-V9.0 (format ff "~2%(require :acache \"acache-3.0.6.fasl\")
;;; File for loading MOSS or OMAS automatically from init window")
    (format ff "~2%;;; change console position and size
(set-console-exterior 0 400 (width (screen *system*))
                      (- (height (screen *system*)) (+ 400 30)))

;;; loading OMAS by default, MOSS if CONTROL key is hold down, nothing if ALT key is
;;; hold down

(cond 
 ((cg:key-is-down-p cg:vk-alt))
 ((cg:key-is-down-p cg:vk-control)
  (load
   (make-pathname 
    :host (pathname-host *load-pathname*)
    :device (pathname-device *load-pathname*)
    :directory (append (pathname-directory *load-pathname*)
                       (list *omas-folder* \"MOSS\"))
    :name \"moss-load\"
    :type \"fasl\")))
 (t
  (load
   (make-pathname 
    :host (pathname-host *load-pathname*)
    :device (pathname-device *load-pathname*)
    :directory (append (pathname-directory *load-pathname*)
                       (list *omas-folder*))
    :name \"load-omas-moss\"
    :type \"fasl\")))
 )"
      ))
  t)

;;; start OMAS file

(omas-install)

;;;--- EOF
