;;;-*- Mode: Lisp; Package: "CG-USER" -*-
;;;===============================================================================
;;;13/04/08
;;;            O M A S - U P G R A D E omas-(upgrade.lisp)
;;;
;;;                copyright JP Barthes @UTC, 2011-2013
;;;
;;;===============================================================================

#| This file contains a function for upgrading to a new version. Only valid for ACL
on Windows.
The difference with the install file is that it extracts the MOSS and OMAS
applications files from the old OMAS-MOSS version.
   
Upgrading to a new version
--------------------------
Perform the following steps:
1. Load the new released version into the same folder as the previous one.
2. Start ACL which should load current version of OMAS. If not, load current version
   of OMAS
3. Go to the File menu, select the "compile and load ..." entry
   Select the "upgrade.lisp" file in the OMAS-MOSS folder
4. You will be asked to select the folder of the new released version
5. You will be asked to allow clobbering the startup.cl file. Answer yes, unless 
   you are using it for launching things other than OMAS
6. The rest should be automatic. For your information, your MOSS and OMAS 
   application files will be transferred to the new release, and the startup.cl 
   file will be upgraded, so that next time you start Lisp the new version of OMAS
   is started automatically
7. Exit
8. Check that your applications have been transferred.
9. Restart ACL. It should boot on the new version and your applications folders 
   should contain your applications.

2011
 1207 creation
2012
 0505 bug in copy of startup.cl file text, and in path to omas.parameters
 0506 adding a test to omas-upgrade
2013
 0408 correcting acache bug in create-startup-file
|#

#-OMAS
(error "you should be running OMAS to upgrade to a new version")

;;; We stay in the user package for MCL and the CG-USR package for ACL
(in-package :cg-user)

;;;================================== Globals ====================================
;;; we need some globals
;;; pathname to the ACL 8.2 (Home) folder
(defparameter *home-pathname* (excl::get-home-location) "pathname to ACL folder")

;;; pathname to the startup.cl file: NIL if it does not exist
(defparameter *acl-startup-file* 
     (merge-pathnames "startup.cl" *home-pathname*))	
    
;;; pathname to the OMAS startup.cl file present at the top level
(defparameter *omas-startup-file*
    (make-pathname
     :host (pathname-host omas::*omas-directory-string*)
     :device (pathname-device omas::*omas-directory-string*)
     :directory (butlast (pathname-directory omas::*omas-directory-string*)) 
     :name "startup.cl"))

;; pathname to the current folder of applications
(defparameter *omas-applications*
  (make-pathname
   :host (pathname-host omas::*omas-directory-string*)
   :device (pathname-device omas::*omas-directory-string*)
   :directory (append (pathname-directory omas::*omas-directory-string*) 
                              '( "applications"))))

;; pathname to the omas.parameters file
(defparameter *omas-properties*
  (make-pathname
     :host (pathname-host omas::*omas-directory-string*)
     :device (pathname-device omas::*omas-directory-string*)
     :directory (butlast (pathname-directory omas::*omas-directory-string*))
     :name "omas" :type "properties"))

;; id. for MOSS
(defparameter *moss-applications*
  (make-pathname
   :host (pathname-host moss::*moss-directory-string*)
   :device (pathname-device moss::*moss-directory-string*)
   :directory (append (pathname-directory moss::*moss-directory-string*) 
                              '( "applications"))))

;; flag to block the init window
;(defparameter *no-init-window* t "block the init window")
	
;;;============================== macros =========================================
;;; none

;;;============================= functions =======================================
	 
;;;------------------------------------------------------------------ OMAS-UPGRADE
;;; the function requires the transfer of application files, clobbering
;;; the default ones

(defun omas-upgrade ()
  "for upgrading to a new OMAS version"
  (declare (special *acl-startup-file* *omas-startup-file* *omas-properties*
                    *moss-applications* *omas-applications*))
  (let (new-directory new-moss-pathname new-omas-pathname 
                      new-omas-properties-pathname)
    
    ;; locate folder of the new release (the :root option does not work on the virtual
    ;; machine for some reason)
    (setq new-directory 
          (ask-user-for-directory 
           :prompt "Please select the folder of the new Release."))
    (unless new-directory (return-from omas-upgrade))
    ;; build MOSS directory pathname
    (setq new-MOSS-pathname 
          (make-pathname 
           :host (pathname-host new-directory)
           :device (pathname-device new-directory)
           :directory (append (pathname-directory new-directory) 
                              '("MOSS" "applications"))))
    ;; same for OMAS applications folder
    (setq new-OMAS-pathname 
          (make-pathname 
           :host (pathname-host new-directory)
           :device (pathname-device new-directory)
           :directory (append (pathname-directory new-directory) 
                              '("OMAS" "applications"))))	
    
    
    ;; fetch the applications folders from the old version and move them to the
    ;; new one
    (excl:copy-directory *moss-applications* new-MOSS-pathname :overwrite t)
    (excl:copy-directory *omas-applications* new-OMAS-pathname :overwrite t)

    ;; build a pathname for the new omas.properties file. This keeps the current OMAS
    ;; parameters rather that reinitializing them
    (setq new-OMAS-properties-pathname 
          (make-pathname 
           :host (pathname-host new-directory)
           :device (pathname-device new-directory)
           :directory (pathname-directory new-directory)
           :name "omas"
           :type "properties"))

    ;; copy and overwrite the omas.properties file
    (sys:copy-file *omas-properties* new-OMAS-properties-pathname :overwrite t)
    
    ;; copy the startup default file
    (cond
     ;; if no startup file in ACL create a new one
     ((null (probe-file *acl-startup-file*))
      (create-startup-file new-directory))
     ;; if there is already a file ask for replacement
     ((y-or-n-p 
       "A startup.cl file already exists. Do you want to overwrite it? If not, ~
        then you will have to update the file manually to boot OMAS automatically."
       )
      (create-startup-file new-directory)))
    ;; exit
    :done))

;;;----------------------------------------------------------- CREATE-STARTUP-FILE

(defun create-startup-file (directory)
  "creates a file in the ACL folder, provided that the OMAS folder is there"
  ;; create a simple startup file
  (with-open-file (ff "startup.cl" :if-does-not-exist :create
                      :if-exists :supersede
                      :direction :output)
    (format ff ";;; File for loading MOSS or OMAS automatically from init window")
    (format ff "~2%(defparameter *OMAS-folder* ~S)" 
      (car (last (pathname-directory directory))))
    #+ALLEGRO-V9.0 (format ff "~2%(require :acache \"acache-3.0.6.fasl\")
;;; File for loading MOSS or OMAS automatically from init window")
    (format ff "~2%;;; change console position and size
(set-console-exterior 0 400 (width (screen *system*))
                      (- (height (screen *system*)) (+ 400 30)))

;;; loading OMAS by default, MOSS if CONTROL key is hold down, nothing if ALT key is
;;; hold down

(cond 
 ((cg:key-is-down-p cg:vk-alt))
 ((cg:key-is-down-p cg:vk-control)
  (load
   (make-pathname 
    :host (pathname-host *load-pathname*)
    :device (pathname-device *load-pathname*)
    :directory (append (pathname-directory *load-pathname*)
                       (list *omas-folder* \"MOSS\"))
    :name \"moss-load\"
    :type \"fasl\")))
 (t
  (load
   (make-pathname 
    :host (pathname-host *load-pathname*)
    :device (pathname-device *load-pathname*)
    :directory (append (pathname-directory *load-pathname*)
                       (list *omas-folder*))
    :name \"load-omas-moss\"
    :type \"fasl\")))
 )"))
  t)


;;; launch the upgrade

(omas-upgrade)

;;;--- EOF
